""" 
Numerical Chern number calculation according to
Fukui et al. J. Phys. Soc. Jpn. 74, 1674 (2005)

:author:  $Sebastian Huber   $:
:version: $Revision:: 133    $:
:date:    $Date:: 2014-02-26#$:

This modules provides the classes

- :class:`~chernsimulations.chern`
- :class:`~chernsimulations.chernSimple`

.. inheritance-diagram:: chernsimulations

"""
import numpy as np
import scipy.linalg as sl
import pickle
import matplotlib.pyplot as plt

class chernSimple():
	""" Simple Chern number calculator
	
	Notes
	-----
	
	The calculation requires that there are files called
	X_Y.npy with X,Y ranging from 0 to grid-1
	in the directory of the script using this class
	
	"""
	def __init__(self):
		version = filter(str.isdigit, "$Revision: 133 $")
		print "  Using revision: %s of chernSimple class" %(version)
		
		
	
	def calculateChern(self,param=None):
		""" Return the calculated Chern number
		 
		Parameters
		----------
		
		param: dictionary {'n': 1, 'grid' 5, 'fieldstrength' False}
		 n: int 
		  number of states to form a Slater determinant. Use n=1 for bosonic systems
		 
		 grid: int
		  How many states did we calculate
		 
		 fieldstrength: Boolean
		  Show field strength plot
		
		Examples
		========
		
		>>> 
		
		"""
		self.fs=False
		if param is not None:
			n=param.get('n',1)
			self.grid=param.get('grid',10)
			self.fs = param.get('fieldstrength',False)
		else:
			n=1
			self.grid=5
		
		# Link variables
		ux = np.zeros((self.grid,self.grid),dtype=np.complex128)
		uy = np.zeros((self.grid,self.grid),dtype=np.complex128)
		# Constructing link variables in x-direction
		for i in range(self.grid):
			for j in range(self.grid-1):
				filename = "%0d_%0d.npy" % (j,i)
				left=np.mat(np.load(filename))
				filename = "%0d_%0d.npy" % (j+1,i)
				right=np.mat(np.load(filename))
				ux[j,i] = sl.det(left[:,0:n].H*right[:,0:n])
				ux[j,i] = ux[j,i]/abs(ux[j,i])
		
		# Constructing link variables in y-direction
		for i in range(self.grid-1):
			for j in range(self.grid):
				filename = "%0d_%0d.npy" % (j,i)
				left=np.mat(np.load(filename))
				filename = "%0d_%0d.npy" % (j,i+1)
				right=np.mat(np.load(filename))
				uy[j,i] = sl.det(left[:,0:n].H*right[:,0:n])
				uy[j,i] = uy[j,i]/abs(uy[j,i])
		
		# Field strength
		f = np.zeros((self.grid-1,self.grid-1))
		for i in range(self.grid-1):
			for j in range(self.grid-1):
				f[i,j]=np.imag(np.log(ux[i,j]*uy[i+1][j]/ux[i][j+1]/uy[i,j]))
		
		if self.fs:
			plt.imshow(f,interpolation='nearest')
			plt.show()
		
		# Calculate Chern number
		return np.around(np.sum(f)/(2*np.pi),8)